@@ -1,7 +1,7 @@
 /*
 	DDS GIMP plugin
 
-	Copyright (C) 2004 Shawn Kirst <skirst@fuse.net>,
+	Copyright (C) 2004 Shawn Kirst <skirst@gmail.com>,
    with parts (C) 2003 Arne Reuter <homepage@arnereuter.de> where specified.
 
 	This program is free software; you can redistribute it and/or
@@ -16,8 +16,8 @@
 
 	You should have received a copy of the GNU General Public License
 	along with this program; see the file COPYING.  If not, write to
-	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-	Boston, MA 02111-1307, USA.
+   the Free Software Foundation, 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301 USA.
 */
 
 /*
@@ -41,6 +41,8 @@
 #include "dds.h"
 #include "dxt.h"
 #include "endian.h"
+#include "misc.h"
+#include "imath.h"
 
 typedef struct
 {
@@ -53,6 +55,7 @@ typedef struct
 } dds_load_info_t;
 
 static int read_header(dds_header_t *hdr, FILE *fp);
+static int read_header_dx10(dds_header_dx10_t *hdr, FILE *fp);
 static int validate_header(dds_header_t *hdr);
 static int load_layer(FILE *fp, dds_header_t *hdr, dds_load_info_t *d,
                       gint32 image, unsigned int level, char *prefix,
@@ -65,8 +68,11 @@ static int load_face(FILE *fp, dds_header_t *hdr, dds_load_info_t *d,
                      guchar *pixels, unsigned char *buf);
 static unsigned char color_bits(unsigned int mask);
 static unsigned char color_shift(unsigned int mask);
+static int load_dialog(void);
 
-gint32 read_dds(gchar *filename)
+static int runme = 0;
+
+GimpPDBStatusType read_dds(gchar *filename, gint32 *imageID)
 {
    gint32 image = 0;
    unsigned char *buf;
@@ -75,18 +81,25 @@ gint32 read_dds(gchar *filename)
    gchar *tmp;
    FILE *fp;
    dds_header_t hdr;
+   dds_header_dx10_t dx10hdr;
    dds_load_info_t d;
    gint *layers, layer_count;
    GimpImageBaseType type;
    int i, j;
-   
+
+   if(interactive_dds && dds_read_vals.show_dialog)
+   {
+      if(!load_dialog())
+         return(GIMP_PDB_CANCEL);
+   }
+
    fp = fopen(filename, "rb");
    if(fp == 0)
    {
       g_message("Error opening file.\n");
-      return(-1);
+      return(GIMP_PDB_EXECUTION_ERROR);
    }
-   
+
    if(interactive_dds)
    {
       if(strrchr(filename, '/'))
@@ -96,14 +109,15 @@ gint32 read_dds(gchar *filename)
       gimp_progress_init(tmp);
       g_free(tmp);
    }
-   
+
    read_header(&hdr, fp);
    if(!validate_header(&hdr))
    {
       fclose(fp);
-      return(-1);
+      g_message("Invalid DDS header!\n");
+      return(GIMP_PDB_EXECUTION_ERROR);
    }
-   
+
    /* a lot of DDS images out there don't have this for some reason -_- */
    if(hdr.pitch_or_linsize == 0)
    {
@@ -121,106 +135,125 @@ gint32 read_dds(gchar *filename)
             (hdr.pixelfmt.bpp >> 3);
       }
    }
-   
-   if(hdr.pixelfmt.flags & DDPF_FOURCC)
-   {      
-      if(hdr.pixelfmt.fourcc[0] == 'D')
-         hdr.pixelfmt.flags |= DDPF_ALPHAPIXELS;
-   }
-   
-   if(hdr.pixelfmt.flags & DDPF_FOURCC)
+
+   if(GETL32(hdr.pixelfmt.fourcc) == FOURCC('D', 'X', '1', '0'))
    {
-      switch(GETL32(hdr.pixelfmt.fourcc))
-      {
-         case CHAR32('A', 'T', 'I', '1'):
-            d.bpp = d.gimp_bpp = 1;
-            type = GIMP_GRAY;
-            break;
-         case CHAR32('A', 'T', 'I', '2'):
-            d.bpp = d.gimp_bpp = 3;
-            type = GIMP_RGB;
-            break;
-         default: 
-            d.bpp = d.gimp_bpp = 4;
-            type = GIMP_RGB;
-            break;
-      }
+      read_header_dx10(&dx10hdr, fp);
+
+      /* TODO: Support DX10 DDS extensions */
+
+      fclose(fp);
+      g_message("DX10 images not yet supported!\n");
+      return(GIMP_PDB_EXECUTION_ERROR);
    }
    else
    {
-      d.bpp = hdr.pixelfmt.bpp >> 3;
-
-      if(d.bpp == 2)
+      if(hdr.pixelfmt.flags & DDPF_FOURCC)
       {
-         if(hdr.pixelfmt.amask == 0xf000) // RGBA4
+         if(hdr.pixelfmt.fourcc[1] == 'X')
+            hdr.pixelfmt.flags |= DDPF_ALPHAPIXELS;
+      }
+
+      if(hdr.pixelfmt.flags & DDPF_FOURCC)
+      {
+         switch(GETL32(hdr.pixelfmt.fourcc))
          {
-            d.gimp_bpp = 4;
-            type = GIMP_RGB;
-         }
-         else if(hdr.pixelfmt.amask == 0xff00) //L8A8
-         {
-            d.gimp_bpp = 2;
-            type = GIMP_GRAY;
-         }
-         else if(hdr.pixelfmt.bmask == 0x1f) //R5G6B5 or RGB5A1
-         {
-            if(hdr.pixelfmt.amask == 0x8000) // RGB5A1
-               d.gimp_bpp = 4;
-            else
-               d.gimp_bpp = 3;
-            
-            type = GIMP_RGB;
-         }
-         else //L16
-         {
-            d.gimp_bpp = 1;
-            type = GIMP_GRAY;
+            case FOURCC('A', 'T', 'I', '1'):
+               d.bpp = d.gimp_bpp = 1;
+               type = GIMP_GRAY;
+               break;
+            case FOURCC('A', 'T', 'I', '2'):
+               d.bpp = d.gimp_bpp = 3;
+               type = GIMP_RGB;
+               break;
+            default:
+               d.bpp = d.gimp_bpp = 4;
+               type = GIMP_RGB;
+               break;
          }
       }
       else
       {
-         if(hdr.pixelfmt.flags & DDPF_PALETTEINDEXED8)
+         d.bpp = hdr.pixelfmt.bpp >> 3;
+
+         if(d.bpp == 2)
          {
-            type = GIMP_INDEXED;
-            d.gimp_bpp = 1;
-         }
-         else if(hdr.pixelfmt.rmask == 0xe0) // R3G3B2
-         {
-            type = GIMP_RGB;
-            d.gimp_bpp = 3;
-         }
-         else
-         {
-            /* test alpha only image */
-            if(d.bpp == 1 && (hdr.pixelfmt.flags & DDPF_ALPHA))
+            if(hdr.pixelfmt.amask == 0xf000) // RGBA4
+            {
+               d.gimp_bpp = 4;
+               type = GIMP_RGB;
+            }
+            else if(hdr.pixelfmt.amask == 0xff00) //L8A8
             {
                d.gimp_bpp = 2;
                type = GIMP_GRAY;
             }
+            else if(hdr.pixelfmt.bmask == 0x1f) //R5G6B5 or RGB5A1
+            {
+               if(hdr.pixelfmt.amask == 0x8000) // RGB5A1
+                  d.gimp_bpp = 4;
+               else
+                  d.gimp_bpp = 3;
+
+               type = GIMP_RGB;
+            }
+            else //L16
+            {
+               d.gimp_bpp = 1;
+               type = GIMP_GRAY;
+            }
+         }
+         else
+         {
+            if(hdr.pixelfmt.flags & DDPF_PALETTEINDEXED8)
+            {
+               type = GIMP_INDEXED;
+               d.gimp_bpp = 1;
+            }
+            else if(hdr.pixelfmt.rmask == 0xe0) // R3G3B2
+            {
+               type = GIMP_RGB;
+               d.gimp_bpp = 3;
+            }
             else
             {
-               d.gimp_bpp = d.bpp;
-               type = (d.bpp == 1) ? GIMP_GRAY : GIMP_RGB;
+               /* test alpha only image */
+               if(d.bpp == 1 && (hdr.pixelfmt.flags & DDPF_ALPHA))
+               {
+                  d.gimp_bpp = 2;
+                  type = GIMP_GRAY;
+               }
+               else
+               {
+                  d.gimp_bpp = d.bpp;
+                  type = (d.bpp == 1) ? GIMP_GRAY : GIMP_RGB;
+               }
             }
          }
       }
    }
-   
+
    image = gimp_image_new(hdr.width, hdr.height, type);
-   
+
    if(image == -1)
    {
       g_message("Can't allocate new image.\n");
       fclose(fp);
-      return(-1);
+      return(GIMP_PDB_EXECUTION_ERROR);
    }
-   
+
    gimp_image_set_filename(image, filename);
-   
+
    if(hdr.pixelfmt.flags & DDPF_PALETTEINDEXED8)
    {
       d.palette = g_malloc(256 * 4);
-      fread(d.palette, 1, 256 * 4, fp);
+      if(fread(d.palette, 1, 1024, fp) != 1024)
+      {
+         g_message("Error reading palette.\n");
+         fclose(fp);
+         gimp_image_delete(image);
+         return(GIMP_PDB_EXECUTION_ERROR);
+      }
       for(i = j = 0; i < 768; i += 3, j += 4)
       {
          d.palette[i + 0] = d.palette[j + 0];
@@ -229,12 +262,12 @@ gint32 read_dds(gchar *filename)
       }
       gimp_image_set_colormap(image, d.palette, 256);
    }
-   
+
    d.tile_height = gimp_tile_height();
-   
+
    pixels = g_new(guchar, d.tile_height * hdr.width * d.gimp_bpp);
    buf = g_malloc(hdr.pitch_or_linsize);
-   
+
    d.rshift = color_shift(hdr.pixelfmt.rmask);
    d.gshift = color_shift(hdr.pixelfmt.gmask);
    d.bshift = color_shift(hdr.pixelfmt.bmask);
@@ -247,7 +280,7 @@ gint32 read_dds(gchar *filename)
    d.gmask = hdr.pixelfmt.gmask >> d.gshift << (8 - d.gbits);
    d.bmask = hdr.pixelfmt.bmask >> d.bshift << (8 - d.bbits);
    d.amask = hdr.pixelfmt.amask >> d.ashift << (8 - d.abits);
-   
+
    if(!(hdr.caps.caps2 & DDSCAPS2_CUBEMAP) &&
       !(hdr.caps.caps2 & DDSCAPS2_VOLUME))
    {
@@ -255,102 +288,99 @@ gint32 read_dds(gchar *filename)
       {
          fclose(fp);
          gimp_image_delete(image);
-         return(-1);
+         return(GIMP_PDB_EXECUTION_ERROR);
       }
-   }
-
-   if(hdr.caps.caps1 & DDSCAPS_COMPLEX)
-   {
-      if(hdr.caps.caps2 & DDSCAPS2_CUBEMAP)
-      {
-         if((hdr.caps.caps2 & DDSCAPS2_CUBEMAP_POSITIVEX) &&
-            !load_face(fp, &hdr, &d, image, "(positive x)", &l, pixels, buf))
-         {
-            fclose(fp);
-            gimp_image_delete(image);
-            return(-1);
-         }
-         if((hdr.caps.caps2 & DDSCAPS2_CUBEMAP_NEGATIVEX) &&
-            !load_face(fp, &hdr, &d, image, "(negative x)", &l, pixels, buf))
-         {
-            fclose(fp);
-            gimp_image_delete(image);
-            return(-1);
-         }
-         if((hdr.caps.caps2 & DDSCAPS2_CUBEMAP_POSITIVEY) &&
-            !load_face(fp, &hdr, &d, image, "(positive y)", &l, pixels, buf))
-         {
-            fclose(fp);
-            gimp_image_delete(image);
-            return(-1);
-         }
-         if((hdr.caps.caps2 & DDSCAPS2_CUBEMAP_NEGATIVEY) &&
-            !load_face(fp, &hdr, &d, image, "(negative y)", &l, pixels, buf))
-         {
-            fclose(fp);
-            gimp_image_delete(image);
-            return(-1);
-         }
-         if((hdr.caps.caps2 & DDSCAPS2_CUBEMAP_POSITIVEZ) &&
-            !load_face(fp, &hdr, &d, image, "(positive z)", &l, pixels, buf))
-         {
-            fclose(fp);
-            gimp_image_delete(image);
-            return(-1);
-         }
-         if((hdr.caps.caps2 & DDSCAPS2_CUBEMAP_NEGATIVEZ) &&
-            !load_face(fp, &hdr, &d, image, "(negative z)", &l, pixels, buf))
-         {
-            fclose(fp);
-            gimp_image_delete(image);
-            return(-1);
-         }
-      }
-      else if((hdr.caps.caps2 & DDSCAPS2_VOLUME) &&
-              (hdr.flags & DDSD_DEPTH))
-      {
-         unsigned int i, level;
-         char *plane;
-         for(i = 0; i < hdr.depth; ++i)
-         {
-            plane = g_strdup_printf("(z = %d)", i);
-            if(!load_layer(fp, &hdr, &d, image, 0, plane, &l, pixels, buf))
-            {
-               g_free(plane);
-               fclose(fp);
-               gimp_image_delete(image);
-               return(-1);
-            }
-            g_free(plane);
-         }
-
-         if((hdr.flags & DDSD_MIPMAPCOUNT) &&
-            (hdr.caps.caps1 & DDSCAPS_MIPMAP))
-         {
-            for(level = 1; level < hdr.num_mipmaps; ++level)
-            {
-               int n = hdr.depth >> level;
-               if(n < 1) n = 1;
-               for(i = 0; i < n; ++i)
-               {
-                  plane = g_strdup_printf("(z = %d)", i);
-                  if(!load_layer(fp, &hdr, &d, image, level, plane, &l, pixels, buf))
-                  {
-                     g_free(plane);
-                     fclose(fp);
-                     gimp_image_delete(image);
-                     return(-1);
-                  }
-                  g_free(plane);
-               }
-            }
-         }
-      }
-      else if(!load_mipmaps(fp, &hdr, &d, image, "", &l, pixels, buf))
+      if(!load_mipmaps(fp, &hdr, &d, image, "", &l, pixels, buf))
       {
          fclose(fp);
          gimp_image_delete(image);
-         return(-1);
+         return(GIMP_PDB_EXECUTION_ERROR);
+      }
+   }
+   else if(hdr.caps.caps2 & DDSCAPS2_CUBEMAP)
+   {
+      if((hdr.caps.caps2 & DDSCAPS2_CUBEMAP_POSITIVEX) &&
+         !load_face(fp, &hdr, &d, image, "(positive x)", &l, pixels, buf))
+      {
+         fclose(fp);
+         gimp_image_delete(image);
+         return(GIMP_PDB_EXECUTION_ERROR);
+      }
+      if((hdr.caps.caps2 & DDSCAPS2_CUBEMAP_NEGATIVEX) &&
+         !load_face(fp, &hdr, &d, image, "(negative x)", &l, pixels, buf))
+      {
+         fclose(fp);
+         gimp_image_delete(image);
+         return(GIMP_PDB_EXECUTION_ERROR);
+      }
+      if((hdr.caps.caps2 & DDSCAPS2_CUBEMAP_POSITIVEY) &&
+         !load_face(fp, &hdr, &d, image, "(positive y)", &l, pixels, buf))
+      {
+         fclose(fp);
+         gimp_image_delete(image);
+         return(GIMP_PDB_EXECUTION_ERROR);
+      }
+      if((hdr.caps.caps2 & DDSCAPS2_CUBEMAP_NEGATIVEY) &&
+         !load_face(fp, &hdr, &d, image, "(negative y)", &l, pixels, buf))
+      {
+         fclose(fp);
+         gimp_image_delete(image);
+         return(GIMP_PDB_EXECUTION_ERROR);
+      }
+      if((hdr.caps.caps2 & DDSCAPS2_CUBEMAP_POSITIVEZ) &&
+         !load_face(fp, &hdr, &d, image, "(positive z)", &l, pixels, buf))
+      {
+         fclose(fp);
+         gimp_image_delete(image);
+         return(GIMP_PDB_EXECUTION_ERROR);
+      }
+      if((hdr.caps.caps2 & DDSCAPS2_CUBEMAP_NEGATIVEZ) &&
+         !load_face(fp, &hdr, &d, image, "(negative z)", &l, pixels, buf))
+      {
+         fclose(fp);
+         gimp_image_delete(image);
+         return(GIMP_PDB_EXECUTION_ERROR);
+      }
+   }
+   else if((hdr.caps.caps2 & DDSCAPS2_VOLUME) &&
+            (hdr.flags & DDSD_DEPTH))
+   {
+      unsigned int i, level;
+      char *plane;
+      for(i = 0; i < hdr.depth; ++i)
+      {
+         plane = g_strdup_printf("(z = %d)", i);
+         if(!load_layer(fp, &hdr, &d, image, 0, plane, &l, pixels, buf))
+         {
+            g_free(plane);
+            fclose(fp);
+            gimp_image_delete(image);
+            return(GIMP_PDB_EXECUTION_ERROR);
+         }
+         g_free(plane);
+      }
+
+      if((hdr.flags & DDSD_MIPMAPCOUNT) &&
+         (hdr.caps.caps1 & DDSCAPS_MIPMAP) &&
+         (dds_read_vals.mipmaps != 0))
+      {
+         for(level = 1; level < hdr.num_mipmaps; ++level)
+         {
+            int n = hdr.depth >> level;
+            if(n < 1) n = 1;
+            for(i = 0; i < n; ++i)
+            {
+               plane = g_strdup_printf("(z = %d)", i);
+               if(!load_layer(fp, &hdr, &d, image, level, plane, &l, pixels, buf))
+               {
+                  g_free(plane);
+                  fclose(fp);
+                  gimp_image_delete(image);
+                  return(GIMP_PDB_EXECUTION_ERROR);
+               }
+               g_free(plane);
+            }
+         }
       }
    }
 
@@ -362,25 +392,34 @@ gint32 read_dds(gchar *filename)
    fclose(fp);
 
    layers = gimp_image_get_layers(image, &layer_count);
+
+   if(layers == NULL || layer_count == 0)
+   {
+      g_message("Oops!  NULL image read!  Please report this!");
+      return(GIMP_PDB_EXECUTION_ERROR);
+   }
+
    gimp_image_set_active_layer(image, layers[0]);
-               
-   return(image);
+
+   *imageID = image;
+
+   return(GIMP_PDB_SUCCESS);
 }
 
 static int read_header(dds_header_t *hdr, FILE *fp)
 {
    unsigned char buf[DDS_HEADERSIZE];
-   
+
    memset(hdr, 0, sizeof(dds_header_t));
-   
+
    if(fread(buf, 1, DDS_HEADERSIZE, fp) != DDS_HEADERSIZE)
       return(0);
-   
+
    hdr->magic[0] = buf[0];
    hdr->magic[1] = buf[1];
    hdr->magic[2] = buf[2];
    hdr->magic[3] = buf[3];
-   
+
    hdr->size = GETL32(buf + 4);
    hdr->flags = GETL32(buf + 8);
    hdr->height = GETL32(buf + 12);
@@ -388,7 +427,7 @@ static int read_header(dds_header_t *hdr, FILE *fp)
    hdr->pitch_or_linsize = GETL32(buf + 20);
    hdr->depth = GETL32(buf + 24);
    hdr->num_mipmaps = GETL32(buf + 28);
-   
+
    hdr->pixelfmt.size = GETL32(buf + 76);
    hdr->pixelfmt.flags = GETL32(buf + 80);
    hdr->pixelfmt.fourcc[0] = buf[84];
@@ -400,21 +439,51 @@ static int read_header(dds_header_t *hdr, FILE *fp)
    hdr->pixelfmt.gmask = GETL32(buf + 96);
    hdr->pixelfmt.bmask = GETL32(buf + 100);
    hdr->pixelfmt.amask = GETL32(buf + 104);
-   
+
    hdr->caps.caps1 = GETL32(buf + 108);
    hdr->caps.caps2 = GETL32(buf + 112);
-   
+
+   /* GIMP-DDS special info */
+   if(GETL32(buf + 32) == FOURCC('G','I','M','P') &&
+      GETL32(buf + 36) == FOURCC('-','D','D','S'))
+   {
+      hdr->reserved.gimp_dds_special.magic1 = GETL32(buf + 32);
+      hdr->reserved.gimp_dds_special.magic2 = GETL32(buf + 36);
+      hdr->reserved.gimp_dds_special.version = GETL32(buf + 40);
+      hdr->reserved.gimp_dds_special.extra_fourcc = GETL32(buf + 44);
+   }
+
+   return(1);
+}
+
+static int read_header_dx10(dds_header_dx10_t *hdr, FILE *fp)
+{
+   char buf[DDS_HEADERSIZE_DX10];
+
+   memset(hdr, 0, sizeof(dds_header_dx10_t));
+
+   if(fread(buf, 1, DDS_HEADERSIZE_DX10, fp) != DDS_HEADERSIZE_DX10)
+      return(0);
+
+   hdr->dxgiFormat = GETL32(buf);
+   hdr->resourceDimension = GETL32(buf + 4);
+   hdr->miscFlag = GETL32(buf + 8);
+   hdr->arraySize = GETL32(buf + 12);
+   hdr->reserved = GETL32(buf + 16);
+
    return(1);
 }
 
 static int validate_header(dds_header_t *hdr)
 {
+   unsigned int fourcc;
+
    if(memcmp(hdr->magic, "DDS ", 4))
    {
       g_message("Invalid DDS file.\n");
       return(0);
    }
-   
+
    if((hdr->flags & DDSD_PITCH) == (hdr->flags & DDSD_LINEARSIZE))
    {
       //g_message("Warning: DDSD_PITCH or DDSD_LINEARSIZE is not set.\n");
@@ -423,26 +492,29 @@ static int validate_header(dds_header_t *hdr)
       else
          hdr->flags |= DDSD_PITCH;
    }
-/*   
+/*
    if((hdr->pixelfmt.flags & DDPF_FOURCC) ==
       (hdr->pixelfmt.flags & DDPF_RGB))
    {
       g_message("Invalid pixel format.\n");
       return(0);
    }
-*/   
+*/
+   fourcc = GETL32(hdr->pixelfmt.fourcc);
+
    if((hdr->pixelfmt.flags & DDPF_FOURCC) &&
-      memcmp(hdr->pixelfmt.fourcc, "DXT1", 4) &&
-      memcmp(hdr->pixelfmt.fourcc, "DXT3", 4) &&
-      memcmp(hdr->pixelfmt.fourcc, "DXT5", 4) &&
-      memcmp(hdr->pixelfmt.fourcc, "ATI1", 4) &&
-      memcmp(hdr->pixelfmt.fourcc, "ATI2", 4))
+      fourcc != FOURCC('D','X','T','1') &&
+      fourcc != FOURCC('D','X','T','3') &&
+      fourcc != FOURCC('D','X','T','5') &&
+      fourcc != FOURCC('R','X','G','B') &&
+      fourcc != FOURCC('A','T','I','1') &&
+      fourcc != FOURCC('A','T','I','2'))
    {
       g_message("Invalid compression format.\n"
                 "Only DXT1, DXT3, DXT5, ATI1N and ATI2N formats are supported.\n");
       return(0);
    }
-   
+
    if(hdr->pixelfmt.flags & DDPF_RGB)
    {
       if((hdr->pixelfmt.bpp !=  8) &&
@@ -462,27 +534,28 @@ static int validate_header(dds_header_t *hdr)
          g_message("Invalid BPP.\n");
          return(0);
       }
-      
+
       hdr->pixelfmt.flags |= DDPF_RGB;
    }
    else if(hdr->pixelfmt.flags & DDPF_PALETTEINDEXED8)
    {
       hdr->pixelfmt.flags |= DDPF_RGB;
    }
-   
+
    if(!(hdr->pixelfmt.flags & DDPF_RGB) &&
       !(hdr->pixelfmt.flags & DDPF_ALPHA) &&
       !(hdr->pixelfmt.flags & DDPF_FOURCC) &&
       !(hdr->pixelfmt.flags & DDPF_LUMINANCE))
    {
       g_message("Unknown pixel format!  Taking a guess, expect trouble!");
-      switch(GETL32(hdr->pixelfmt.fourcc))
+      switch(fourcc)
       {
-         case CHAR32('D', 'X', 'T', '1'):
-         case CHAR32('D', 'X', 'T', '3'):
-         case CHAR32('D', 'X', 'T', '5'):
-         case CHAR32('A', 'T', 'I', '1'):
-         case CHAR32('A', 'T', 'I', '2'):
+         case FOURCC('D','X','T','1'):
+         case FOURCC('D','X','T','3'):
+         case FOURCC('D','X','T','5'):
+         case FOURCC('R','X','G','B'):
+         case FOURCC('A','T','I','1'):
+         case FOURCC('A','T','I','2'):
             hdr->pixelfmt.flags |= DDPF_FOURCC;
             break;
          default:
@@ -506,7 +579,7 @@ static int validate_header(dds_header_t *hdr)
             break;
       }
    }
-   
+
    return(1);
 }
 
@@ -527,7 +600,7 @@ static int load_layer(FILE *fp, dds_header_t *hdr, dds_load_info_t *d,
 
    if(width < 1) width = 1;
    if(height < 1) height = 1;
-   
+
    switch(d->bpp)
    {
       case 1:
@@ -553,52 +626,56 @@ static int load_layer(FILE *fp, dds_header_t *hdr, dds_load_info_t *d,
       case 3: type = GIMP_RGB_IMAGE;   break;
       case 4: type = GIMP_RGBA_IMAGE;  break;
    }
-   
+
    layer_name = (level) ? g_strdup_printf("mipmap %d %s", level, prefix) :
                           g_strdup_printf("main surface %s", prefix);
-   
+
    layer = gimp_layer_new(image, layer_name, width, height, type, 100,
                           GIMP_NORMAL_MODE);
    g_free(layer_name);
-   
+
+#if GIMP_CHECK_VERSION(2, 8, 0)
+   gimp_image_insert_layer(image, layer, 0, *l);
+#else
    gimp_image_add_layer(image, layer, *l);
+#endif
+   
    if((*l)++) gimp_drawable_set_visible(layer, FALSE);
-   
+
    drawable = gimp_drawable_get(layer);
-   
+
    gimp_pixel_rgn_init(&pixel_region, drawable, 0, 0, drawable->width,
                        drawable->height, TRUE, FALSE);
-   
+
    if(hdr->pixelfmt.flags & DDPF_FOURCC)
    {
       unsigned int w = (width  + 3) >> 2;
       unsigned int h = (height + 3) >> 2;
-      
+
       switch(GETL32(hdr->pixelfmt.fourcc))
       {
-         case CHAR32('D', 'X', 'T', '1'): format = DDS_COMPRESS_BC1; break;
-         case CHAR32('D', 'X', 'T', '3'): format = DDS_COMPRESS_BC2; break;
-         case CHAR32('D', 'X', 'T', '5'): format = DDS_COMPRESS_BC3; break;
-         case CHAR32('A', 'T', 'I', '1'): format = DDS_COMPRESS_BC4; break;
-         case CHAR32('A', 'T', 'I', '2'): format = DDS_COMPRESS_BC5; break;
+         case FOURCC('D','X','T','1'): format = DDS_COMPRESS_BC1; break;
+         case FOURCC('D','X','T','3'): format = DDS_COMPRESS_BC2; break;
+         case FOURCC('D','X','T','5'): format = DDS_COMPRESS_BC3; break;
+         case FOURCC('R','X','G','B'): format = DDS_COMPRESS_BC3; break;
+         case FOURCC('A','T','I','1'): format = DDS_COMPRESS_BC4; break;
+         case FOURCC('A','T','I','2'): format = DDS_COMPRESS_BC5; break;
       }
 
-      if(w == 0) w = 1;
-      if(h == 0) h = 1;
       size = w * h;
       if(format == DDS_COMPRESS_BC1 || format == DDS_COMPRESS_BC4)
          size *= 8;
       else
          size *= 16;
    }
-   
+
    if((hdr->flags & DDSD_LINEARSIZE) &&
       !fread(buf, size, 1, fp))
    {
       g_message("Unexpected EOF.\n");
       return(0);
    }
-   
+
    if(hdr->pixelfmt.flags & DDPF_RGB || hdr->pixelfmt.flags & DDPF_ALPHA)
    {
       z = 0;
@@ -619,14 +696,14 @@ static int load_layer(FILE *fp, dds_header_t *hdr, dds_load_info_t *d,
             g_message("Unexpected EOF.\n");
             return(0);
          }
-         
+
          if(!(hdr->flags & DDSD_LINEARSIZE)) z = 0;
-         
+
          for(x = 0; x < drawable->width; ++x)
          {
             unsigned int pixel = buf[z];
             unsigned int pos = (n * drawable->width + x) * d->gimp_bpp;
-            
+
             if(d->bpp > 1) pixel += ((unsigned int)buf[z + 1] <<  8);
             if(d->bpp > 2) pixel += ((unsigned int)buf[z + 2] << 16);
             if(d->bpp > 3) pixel += ((unsigned int)buf[z + 3] << 24);
@@ -635,9 +712,9 @@ static int load_layer(FILE *fp, dds_header_t *hdr, dds_load_info_t *d,
             {
                if(hdr->pixelfmt.amask == 0xc0000000) // handle RGB10A2
                {
-                  pixels[pos + 0] = (pixel >> d->rshift) >> 2;
+                  pixels[pos + 0] = (pixel >> d->bshift) >> 2;
                   pixels[pos + 1] = (pixel >> d->gshift) >> 2;
-                  pixels[pos + 2] = (pixel >> d->bshift) >> 2;
+                  pixels[pos + 2] = (pixel >> d->rshift) >> 2;
                   if(hdr->pixelfmt.flags & DDPF_ALPHAPIXELS)
                      pixels[pos + 3] = (pixel >> d->ashift << (8 - d->abits) & d->amask) * 255 / d->amask;
                }
@@ -722,14 +799,14 @@ static int load_layer(FILE *fp, dds_header_t *hdr, dds_load_info_t *d,
             z += d->bpp;
          }
       }
-      
+
       gimp_pixel_rgn_set_rect(&pixel_region, pixels, 0, y - n,
                               drawable->width, n);
    }
    else if(hdr->pixelfmt.flags & DDPF_FOURCC)
    {
       unsigned char *dst;
-      
+
       if(!(hdr->flags & DDSD_LINEARSIZE))
       {
          g_message("Image marked as compressed, but DDSD_LINEARSIZE is not set.\n");
@@ -738,16 +815,17 @@ static int load_layer(FILE *fp, dds_header_t *hdr, dds_load_info_t *d,
 
       dst = g_malloc(width * height * d->gimp_bpp);
       memset(dst, 0, width * height * d->gimp_bpp);
-      
+
       if(d->gimp_bpp == 4)
       {
-        for(y = 0; y < height; ++y)
+         for(y = 0; y < height; ++y)
             for(x = 0; x < width; ++x)
-               dst[y * (width * 4) + x + 3] = 255;
+               dst[y * (width * 4) + (x * 4) + 3] = 255;
       }
-         
-      dxt_decompress(dst, buf, format, size, width, height, d->gimp_bpp);
-      
+
+      dxt_decompress(dst, buf, format, size, width, height, d->gimp_bpp,
+                     hdr->pixelfmt.flags & DDPF_NORMAL);
+
       z = 0;
       for(y = 0, n = 0; y < height; ++y, ++n)
       {
@@ -759,21 +837,42 @@ static int load_layer(FILE *fp, dds_header_t *hdr, dds_load_info_t *d,
             if(interactive_dds)
                gimp_progress_update((double)y / (double)hdr->height);
          }
-         
+
          memcpy(pixels + n * drawable->width * d->gimp_bpp,
                 dst + y * drawable->width * d->gimp_bpp,
                 width * d->gimp_bpp);
       }
-      
+
       gimp_pixel_rgn_set_rect(&pixel_region, pixels, 0, y - n,
                               drawable->width, n);
-      
+
       g_free(dst);
    }
-   
+
+   /* gimp dds specific.  decode encoded images */
+   if(dds_read_vals.decode_images &&
+      hdr->reserved.gimp_dds_special.magic1 == FOURCC('G','I','M','P') &&
+      hdr->reserved.gimp_dds_special.magic2 == FOURCC('-','D','D','S'))
+   {
+      switch(hdr->reserved.gimp_dds_special.extra_fourcc)
+      {
+         case FOURCC('A','E','X','P'):
+            decode_alpha_exp_image(drawable->drawable_id);
+            break;
+         case FOURCC('Y','C','G','1'):
+            decode_ycocg_image(drawable->drawable_id);
+            break;
+         case FOURCC('Y','C','G','2'):
+            decode_ycocg_scaled_image(drawable->drawable_id);
+            break;
+         default:
+            break;
+      }
+   }
+
    gimp_drawable_flush(drawable);
    gimp_drawable_detach(drawable);
-   
+
    return(1);
 }
 
@@ -782,9 +881,10 @@ static int load_mipmaps(FILE *fp, dds_header_t *hdr, dds_load_info_t *d,
                         guchar *pixels, unsigned char *buf)
 {
    unsigned int level;
-   
+
    if((hdr->flags & DDSD_MIPMAPCOUNT) &&
-      (hdr->caps.caps1 & DDSCAPS_MIPMAP))
+      (hdr->caps.caps1 & DDSCAPS_MIPMAP) &&
+      (dds_read_vals.mipmaps != 0))
    {
       for(level = 1; level < hdr->num_mipmaps; ++level)
       {
@@ -807,7 +907,7 @@ static int load_face(FILE *fp, dds_header_t *hdr, dds_load_info_t *d,
 static unsigned char color_bits(unsigned int mask)
 {
    unsigned char i = 0;
-   
+
    while(mask)
    {
       if(mask & 1) ++i;
@@ -819,8 +919,81 @@ static unsigned char color_bits(unsigned int mask)
 static unsigned char color_shift(unsigned int mask)
 {
    unsigned char i = 0;
-   
+
    if(!mask) return(0);
    while(!((mask >> i) & 1)) ++i;
    return(i);
 }
+
+static void load_dialog_response(GtkWidget *widget, gint response_id,
+                                 gpointer data)
+{
+   switch(response_id)
+   {
+      case GTK_RESPONSE_OK:
+         runme = 1;
+      default:
+         gtk_widget_destroy(widget);
+         break;
+   }
+}
+
+static void toggle_clicked(GtkWidget *widget, gpointer data)
+{
+   int *flag = (int*)data;
+   (*flag) = !(*flag);
+}
+
+static int load_dialog(void)
+{
+   GtkWidget *dlg;
+   GtkWidget *vbox;
+   GtkWidget *check;
+
+   dlg = gimp_dialog_new("Load DDS", "dds", NULL, GTK_WIN_POS_MOUSE,
+                         gimp_standard_help_func, LOAD_PROC,
+                         GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                         GTK_STOCK_OK, GTK_RESPONSE_OK,
+                         NULL);
+
+   gtk_signal_connect(GTK_OBJECT(dlg), "response",
+                      GTK_SIGNAL_FUNC(load_dialog_response),
+                      0);
+   gtk_signal_connect(GTK_OBJECT(dlg), "destroy",
+                      GTK_SIGNAL_FUNC(gtk_main_quit),
+                      0);
+
+   vbox = gtk_vbox_new(0, 8);
+   gtk_container_set_border_width(GTK_CONTAINER(vbox), 8);
+   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->vbox), vbox, 1, 1, 0);
+   gtk_widget_show(vbox);
+
+   check = gtk_check_button_new_with_label("Load mipmaps");
+   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), dds_read_vals.mipmaps);
+   gtk_signal_connect(GTK_OBJECT(check), "clicked",
+                      GTK_SIGNAL_FUNC(toggle_clicked), &dds_read_vals.mipmaps);
+   gtk_box_pack_start(GTK_BOX(vbox), check, 1, 1, 0);
+   gtk_widget_show(check);
+
+   check = gtk_check_button_new_with_label("Automatically decode YCoCg/AExp images when detected");
+   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), dds_read_vals.decode_images);
+   gtk_signal_connect(GTK_OBJECT(check), "clicked",
+                      GTK_SIGNAL_FUNC(toggle_clicked), &dds_read_vals.decode_images);
+   gtk_box_pack_start(GTK_BOX(vbox), check, 1, 1, 0);
+   gtk_widget_show(check);
+
+   check = gtk_check_button_new_with_label("Show this dialog");
+   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), dds_read_vals.show_dialog);
+   gtk_signal_connect(GTK_OBJECT(check), "clicked",
+                      GTK_SIGNAL_FUNC(toggle_clicked), &dds_read_vals.show_dialog);
+   gtk_box_pack_start(GTK_BOX(vbox), check, 1, 1, 0);
+   gtk_widget_show(check);
+
+   gtk_widget_show(dlg);
+
+   runme = 0;
+
+   gtk_main();
+
+   return(runme);
+}
