@@ -1,7 +1,7 @@
 /*
 	DDS GIMP plugin
 
-	Copyright (C) 2004-2008 Shawn Kirst <skirst@insightbb.com>,
+	Copyright (C) 2004 Shawn Kirst <skirst@gmail.com>,
    with parts (C) 2003 Arne Reuter <homepage@arnereuter.de> where specified.
 
 	This program is free software; you can redistribute it and/or
@@ -16,8 +16,8 @@
 
 	You should have received a copy of the GNU General Public License
 	along with this program; see the file COPYING.  If not, write to
-	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-	Boston, MA 02111-1307, USA.
+   the Free Software Foundation, 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301 USA.
 */
 
 #include <string.h>
@@ -37,30 +37,30 @@ int get_num_mipmaps(int width, int height)
    int w = width << 1;
    int h = height << 1;
    int n = 0;
-   
+
    while(w != 1 || h != 1)
    {
       if(w > 1) w >>= 1;
       if(h > 1) h >>= 1;
       ++n;
    }
-   
+
    return(n);
 }
 
 unsigned int get_mipmapped_size(int width, int height, int bpp,
                                 int level, int num, int format)
 {
-   int w, h, cw, ch, n = 0;
+   int w, h, n = 0;
    unsigned int size = 0;
-   
+
    w = width >> level;
    h = height >> level;
    w = MAX(1, w);
    h = MAX(1, h);
    w <<= 1;
    h <<= 1;
-   
+
    while(n < num && (w != 1 || h != 1))
    {
       if(w > 1) w >>= 1;
@@ -68,14 +68,10 @@ unsigned int get_mipmapped_size(int width, int height, int bpp,
       if(format == DDS_COMPRESS_NONE)
          size += (w * h);
       else
-      {
-         cw = MAX(w, 4);
-         ch = MAX(h, 4);
-         size += ((cw + 3) >> 2) * ((ch + 3) >> 2);
-      }
+         size += ((w + 3) >> 2) * ((h + 3) >> 2);
       ++n;
    }
-   
+
    if(format == DDS_COMPRESS_NONE)
       size *= bpp;
    else
@@ -85,7 +81,7 @@ unsigned int get_mipmapped_size(int width, int height, int bpp,
       else
          size *= 16;
    }
-   
+
    return(size);
 }
 
@@ -95,7 +91,7 @@ unsigned int get_volume_mipmapped_size(int width, int height,
 {
    int w, h, d, n = 0;
    unsigned int size = 0;
-   
+
    w = width >> level;
    h = height >> level;
    d = depth >> level;
@@ -127,10 +123,22 @@ unsigned int get_volume_mipmapped_size(int width, int height,
       else
          size *= 16;
    }
-   
+
    return(size);
 }
 
+int get_next_mipmap_dimensions(int *next_w, int *next_h,
+                               int  curr_w, int  curr_h)
+{
+   if(curr_w == 1 || curr_h == 1)
+      return(0);
+
+   if(next_w) *next_w = curr_w >> 1;
+   if(next_h) *next_h = curr_h >> 1;
+
+   return(1);
+}
+
 static void scale_image_nearest(unsigned char *dst, int dw, int dh,
                                 unsigned char *src, int sw, int sh,
                                 int bpp, int gc, float gamma)
@@ -139,7 +147,7 @@ static void scale_image_nearest(unsigned char *dst, int dw, int dh,
    int ix, iy;
    int srowbytes = sw * bpp;
    int drowbytes = dw * bpp;
-   
+
    for(y = 0; y < dh; ++y)
    {
       iy = (y * sh + sh / 2) / dh;
@@ -178,14 +186,14 @@ static void scale_image_box(unsigned char *dst, int dw, int dh,
       {
          iy = ((y * sh + sh / 2) / dh) - 1;
          if(iy < 0) iy = 0;
-         
+
          for(x = 0; x < dw; ++x)
          {
             ix = ((x * sw + sw / 2) / dw) - 1;
             if(ix < 0) ix = 0;
 
             s = src + (iy * sw + ix) * bpp;
-            
+
             for(n = 0; n < bpp; ++n)
             {
                v = (gamma_correct(s[0], gamma) +
@@ -204,14 +212,14 @@ static void scale_image_box(unsigned char *dst, int dw, int dh,
       {
          iy = ((y * sh + sh / 2) / dh) - 1;
          if(iy < 0) iy = 0;
-         
+
          for(x = 0; x < dw; ++x)
          {
             ix = ((x * sw + sw / 2) / dw) - 1;
             if(ix < 0) ix = 0;
-            
+
             s = src + (iy * sw + ix) * bpp;
-            
+
             for(n = 0; n < bpp; ++n)
             {
                v = (s[0] + s[bpp] + s[sw * bpp] + s[(sw + 1) * bpp]) >> 2;
@@ -231,11 +239,11 @@ static void scale_image_bilinear(unsigned char *dst, int dw, int dh,
    int dstride = dw * bpp;
    unsigned char *s;
    float invgamma;
-   
+
    if(gc)
    {
       invgamma = 1.0 / gamma;
-      
+
       for(y = 0; y < dh; ++y)
       {
          if(dh > 1)
@@ -247,7 +255,7 @@ static void scale_image_bilinear(unsigned char *dst, int dw, int dh,
          }
          else
             iy = wy = 0;
-         
+
          for(x = 0; x < dw; ++x)
          {
             if(dw > 1)
@@ -259,9 +267,9 @@ static void scale_image_bilinear(unsigned char *dst, int dw, int dh,
             }
             else
                ix = wx = 0;
-            
+
             s = src + (iy * sw + ix) * bpp;
-            
+
             for(n = 0; n < bpp; ++n)
             {
                v0 = blerp(gamma_correct(s[0], gamma),
@@ -273,7 +281,7 @@ static void scale_image_bilinear(unsigned char *dst, int dw, int dh,
                ++s;
             }
          }
-      }      
+      }
    }
    else
    {
@@ -288,7 +296,7 @@ static void scale_image_bilinear(unsigned char *dst, int dw, int dh,
          }
          else
             iy = wy = 0;
-         
+
          for(x = 0; x < dw; ++x)
          {
             if(dw > 1)
@@ -300,9 +308,9 @@ static void scale_image_bilinear(unsigned char *dst, int dw, int dh,
             }
             else
                ix = wx = 0;
-            
+
             s = src + (iy * sw + ix) * bpp;
-            
+
             for(n = 0; n < bpp; ++n)
             {
                v0 = blerp(s[0], s[bpp], wx);
@@ -331,7 +339,7 @@ static void scale_image_bicubic(unsigned char *dst, int dw, int dh,
    if(gc)
    {
       invgamma = 1.0 / gamma;
-      
+
       for(y = 0; y < dh; ++y)
       {
          if(dh > 1)
@@ -343,7 +351,7 @@ static void scale_image_bicubic(unsigned char *dst, int dw, int dh,
          }
          else
             iy = wy = 0;
-         
+
          for(x = 0; x < dw; ++x)
          {
             if(dw > 1)
@@ -355,9 +363,9 @@ static void scale_image_bicubic(unsigned char *dst, int dw, int dh,
             }
             else
                ix = wx = 0;
-            
+
             s = src + ((iy - 1) * sw + (ix - 1)) * bpp;
-            
+
             for(n = 0; n < bpp; ++n)
             {
                b = icerp(gamma_correct(s[(sw + 0) * bpp], gamma),
@@ -367,13 +375,13 @@ static void scale_image_bicubic(unsigned char *dst, int dw, int dh,
                if(iy > 0)
                {
                   a = icerp(gamma_correct(s[      0], gamma),
-                            gamma_correct(s[    bpp], gamma), 
+                            gamma_correct(s[    bpp], gamma),
                             gamma_correct(s[2 * bpp], gamma),
                             gamma_correct(s[3 * bpp], gamma), wx);
                }
                else
                   a = b;
-               
+
                c = icerp(gamma_correct(s[(2 * sw + 0) * bpp], gamma),
                          gamma_correct(s[(2 * sw + 1) * bpp], gamma),
                          gamma_correct(s[(2 * sw + 2) * bpp], gamma),
@@ -387,7 +395,7 @@ static void scale_image_bicubic(unsigned char *dst, int dw, int dh,
                }
                else
                   d = c;
-               
+
                v = icerp(a, b, c, d, wy);
                if(v < 0) v = 0;
                if(v > 255) v = 255;
@@ -395,7 +403,7 @@ static void scale_image_bicubic(unsigned char *dst, int dw, int dh,
                ++s;
             }
          }
-      }      
+      }
    }
    else
    {
@@ -410,7 +418,7 @@ static void scale_image_bicubic(unsigned char *dst, int dw, int dh,
          }
          else
             iy = wy = 0;
-         
+
          for(x = 0; x < dw; ++x)
          {
             if(dw > 1)
@@ -422,9 +430,9 @@ static void scale_image_bicubic(unsigned char *dst, int dw, int dh,
             }
             else
                ix = wx = 0;
-            
+
             s = src + ((iy - 1) * sw + (ix - 1)) * bpp;
-            
+
             for(n = 0; n < bpp; ++n)
             {
                b = icerp(s[(sw + 0) * bpp],
@@ -440,7 +448,7 @@ static void scale_image_bicubic(unsigned char *dst, int dw, int dh,
                }
                else
                   a = b;
-               
+
                c = icerp(s[(2 * sw + 0) * bpp],
                          s[(2 * sw + 1) * bpp],
                          s[(2 * sw + 2) * bpp],
@@ -454,7 +462,7 @@ static void scale_image_bicubic(unsigned char *dst, int dw, int dh,
                }
                else
                   d = c;
-               
+
                v = icerp(a, b, c, d, wy);
                if(v < 0) v = 0;
                if(v > 255) v = 255;
@@ -466,7 +474,7 @@ static void scale_image_bicubic(unsigned char *dst, int dw, int dh,
    }
 }
 
-static const float FILTER_RADIUS = 5.0f;
+static const float FILTER_RADIUS = 3.0f;
 
 static float lanczos(float r, float x)
 {
@@ -484,20 +492,20 @@ static void scale_image_lanczos(unsigned char *dst, int dw, int dh,
    const float blur = 1.0f;
    const float xfactor = (float)dw / (float)sw;
    const float yfactor = (float)dh / (float)sh;
-   
+
    int x, y, start, stop, nmax, n, i;
    int sstride = sw * bpp;
    float center, contrib, density, s, r;
-   
+
    unsigned char *d, *row, *col;
-   
+
    float xscale = MIN(xfactor, 1.0f) / blur;
    float yscale = MIN(yfactor, 1.0f) / blur;
    float xsupport = FILTER_RADIUS / xscale;
    float ysupport = FILTER_RADIUS / yscale;
-   
+
    float invgamma;
-   
+
    if(xsupport <= 0.5f)
    {
       xsupport = 0.5f + 1e-12f;
@@ -508,34 +516,34 @@ static void scale_image_lanczos(unsigned char *dst, int dw, int dh,
       ysupport = 0.5f + 1e-12f;
       yscale = 1.0f;
    }
-   
+
    /* resample in Y direction first to temporary buffer */
    unsigned char *tmp;
-   
+
    tmp = g_malloc(sw * dh * bpp);
    d = tmp;
 
    if(gc)
    {
       invgamma = 1.0 / gamma;
-      
+
       for(y = 0; y < dh; ++y)
       {
          for(x = 0; x < sw; ++x)
          {
             col = src + (x * bpp);
-            
+
             center = ((float)y + 0.5f) / yfactor;
             start = (int)MAX(center - ysupport + 0.5f, 0);
             stop = (int)MIN(center + ysupport + 0.5f, sh);
             nmax = stop - start;
             s = (float)start - center + 0.5f;
-               
+
             for(i = 0; i < bpp; ++i)
             {
                density = 0.0f;
                r = 0.0f;
-               
+
                for(n = 0; n < nmax; ++n)
                {
                   contrib = lanczos(FILTER_RADIUS, (s + n) * yscale);
@@ -545,23 +553,23 @@ static void scale_image_lanczos(unsigned char *dst, int dw, int dh,
 
                if(density != 0.0f && density != 1.0f)
                   r /= density;
-               
+
                if(r < 0) r = 0;
                if(r > 255) r = 255;
-               
+
                *d++ = (unsigned char)gamma_correct(r, invgamma);
             }
          }
       }
-      
+
       /* resample temp buffer in X direction */
-      
+
       d = dst;
-      
+
       for(y = 0; y < dh; ++y)
       {
          row = tmp + (y * sstride);
-            
+
          for(x = 0; x < dw; ++x)
          {
             center = ((float)x + 0.5f) / xfactor;
@@ -569,7 +577,7 @@ static void scale_image_lanczos(unsigned char *dst, int dw, int dh,
             stop = (int)MIN(center + xsupport + 0.5f, sw);
             nmax = stop - start;
             s = (float)start - center + 0.5f;
-               
+
             for(i = 0; i < bpp; ++i)
             {
                density = 0.0f;
@@ -584,14 +592,14 @@ static void scale_image_lanczos(unsigned char *dst, int dw, int dh,
 
                if(density != 0.0f && density != 1.0f)
                   r /= density;
-               
+
                if(r < 0) r = 0;
                if(r > 255) r = 255;
-               
+
                *d++ = (unsigned char)gamma_correct(r, invgamma);
             }
          }
-      }      
+      }
    }
    else
    {
@@ -600,18 +608,18 @@ static void scale_image_lanczos(unsigned char *dst, int dw, int dh,
          for(x = 0; x < sw; ++x)
          {
             col = src + (x * bpp);
-            
+
             center = ((float)y + 0.5f) / yfactor;
             start = (int)MAX(center - ysupport + 0.5f, 0);
             stop = (int)MIN(center + ysupport + 0.5f, sh);
             nmax = stop - start;
             s = (float)start - center + 0.5f;
-               
+
             for(i = 0; i < bpp; ++i)
             {
                density = 0.0f;
                r = 0.0f;
-               
+
                for(n = 0; n < nmax; ++n)
                {
                   contrib = lanczos(FILTER_RADIUS, (s + n) * yscale);
@@ -621,23 +629,23 @@ static void scale_image_lanczos(unsigned char *dst, int dw, int dh,
 
                if(density != 0.0f && density != 1.0f)
                   r /= density;
-               
+
                if(r < 0) r = 0;
                if(r > 255) r = 255;
-               
+
                *d++ = (unsigned char)r;
             }
          }
       }
-      
+
       /* resample temp buffer in X direction */
-      
+
       d = dst;
-      
+
       for(y = 0; y < dh; ++y)
       {
          row = tmp + (y * sstride);
-            
+
          for(x = 0; x < dw; ++x)
          {
             center = ((float)x + 0.5f) / xfactor;
@@ -645,7 +653,7 @@ static void scale_image_lanczos(unsigned char *dst, int dw, int dh,
             stop = (int)MIN(center + xsupport + 0.5f, sw);
             nmax = stop - start;
             s = (float)start - center + 0.5f;
-               
+
             for(i = 0; i < bpp; ++i)
             {
                density = 0.0f;
@@ -660,16 +668,16 @@ static void scale_image_lanczos(unsigned char *dst, int dw, int dh,
 
                if(density != 0.0f && density != 1.0f)
                   r /= density;
-               
+
                if(r < 0) r = 0;
                if(r > 255) r = 255;
-               
+
                *d++ = (unsigned char)r;
             }
          }
       }
    }
-   
+
    g_free(tmp);
 }
 
@@ -682,35 +690,35 @@ int generate_mipmaps(unsigned char *dst, unsigned char *src,
    unsigned int sw, sh, dw, dh;
    unsigned char *s, *d;
    mipmapfunc_t mipmap_func = NULL;
-   
+
    if(indexed)
       mipmap_func = scale_image_nearest;
    else
    {
       switch(filter)
       {
-         case DDS_MIPMAP_NEAREST:  mipmap_func = scale_image_nearest;  break;
-         case DDS_MIPMAP_BILINEAR: mipmap_func = scale_image_bilinear; break;
-         case DDS_MIPMAP_BICUBIC:  mipmap_func = scale_image_bicubic;  break;
-         case DDS_MIPMAP_LANCZOS:  mipmap_func = scale_image_lanczos;  break;
-         case DDS_MIPMAP_BOX:
-         default:                  mipmap_func = scale_image_box;      break;
+         case DDS_MIPMAP_FILTER_NEAREST:  mipmap_func = scale_image_nearest;  break;
+         case DDS_MIPMAP_FILTER_BILINEAR: mipmap_func = scale_image_bilinear; break;
+         case DDS_MIPMAP_FILTER_BICUBIC:  mipmap_func = scale_image_bicubic;  break;
+         case DDS_MIPMAP_FILTER_LANCZOS:  mipmap_func = scale_image_lanczos;  break;
+         case DDS_MIPMAP_FILTER_BOX:
+         default:                         mipmap_func = scale_image_box;      break;
       }
    }
-   
+
    memcpy(dst, src, width * height * bpp);
 
    s = dst;
    d = dst + (width * height * bpp);
-   
+
    sw = width;
    sh = height;
-   
+
    for(i = 1; i < mipmaps; ++i)
    {
       dw = MAX(1, sw >> 1);
       dh = MAX(1, sh >> 1);
-  
+
       mipmap_func(d, dw, dh, s, sw, sh, bpp, gc, gamma);
 
       s = d;
@@ -718,7 +726,7 @@ int generate_mipmaps(unsigned char *dst, unsigned char *src,
       sh = dh;
       d += (dw * dh * bpp);
    }
-   
+
    return(1);
 }
 
@@ -765,23 +773,23 @@ static void scale_volume_image_box(unsigned char *dst, int dw, int dh, int dd,
       {
          iz = ((z * sd + sd / 2) / dd) - 1;
          if(iz < 0) iz = 0;
-         
+
          for(y = 0; y < dh; ++y)
          {
             iy = ((y * sh + sh / 2) / dh) - 1;
             if(iy < 0) iy = 0;
-            
+
             for(x = 0; x < dw; ++x)
             {
                ix = ((x * sw + sw / 2) / dw) - 1;
                if(ix < 0) ix = 0;
-               
+
                s1 = src + ((iz * (sw * sh)) + (iy * sw) + ix) * bpp;
                if(iz < dd - 1)
                   s2 = src + (((iz + 1) * (sw * sh)) + (iy * sw) + ix) * bpp;
                else
                   s2 = src;
-               
+
                for(n = 0; n < bpp; ++n)
                {
                   v = (gamma_correct(s1[0], gamma) +
@@ -806,23 +814,23 @@ static void scale_volume_image_box(unsigned char *dst, int dw, int dh, int dd,
       {
          iz = ((z * sd + sd / 2) / dd) - 1;
          if(iz < 0) iz = 0;
-         
+
          for(y = 0; y < dh; ++y)
          {
             iy = ((y * sh + sh / 2) / dh) - 1;
             if(iy < 0) iy = 0;
-            
+
             for(x = 0; x < dw; ++x)
             {
                ix = ((x * sw + sw / 2) / dw) - 1;
                if(ix < 0) ix = 0;
-               
+
                s1 = src + ((iz * (sw * sh)) + (iy * sw) + ix) * bpp;
                if(iz < dd - 1)
                   s2 = src + (((iz + 1) * (sw * sh)) + (iy * sw) + ix) * bpp;
                else
                   s2 = src;
-               
+
                for(n = 0; n < bpp; ++n)
                {
                   v =
@@ -845,7 +853,7 @@ static void scale_volume_image_bilinear(unsigned char *dst, int dw, int dh, int
    int x, y, z, n, ix, iy, iz, wx, wy, wz, v, v0, v1, r0, r1;
    unsigned char *s1, *s2, *d = dst;
    float invgamma;
-   
+
    if(gc)
    {
       invgamma = 1.0 / gamma;
@@ -861,7 +869,7 @@ static void scale_volume_image_bilinear(unsigned char *dst, int dw, int dh, int
          }
          else
             iz = wz = 0;
-         
+
          for(y = 0; y < dh; ++y)
          {
             if(dh > 1)
@@ -873,7 +881,7 @@ static void scale_volume_image_bilinear(unsigned char *dst, int dw, int dh, int
             }
             else
                iy = wy = 0;
-            
+
             for(x = 0; x < dw; ++x)
             {
                if(dw > 1)
@@ -885,10 +893,10 @@ static void scale_volume_image_bilinear(unsigned char *dst, int dw, int dh, int
                }
                else
                   ix = wx = 0;
-               
+
                s1 = src + ((iz * (sw * sh)) + (iy * sw) + ix) * bpp;
                s2 = src + (((iz + 1) * (sw * sh)) + (iy * sw) + ix) * bpp;
-               
+
                for(n = 0; n < bpp; ++n)
                {
                   r0 = blerp(gamma_correct(s1[0], gamma),
@@ -896,13 +904,13 @@ static void scale_volume_image_bilinear(unsigned char *dst, int dw, int dh, int
                   r1 = blerp(gamma_correct(s1[sw * bpp], gamma),
                              gamma_correct(s1[(sw + 1) * bpp], gamma), wx);
                   v0 = blerp(r0, r1, wy);
-                  
+
                   r0 = blerp(gamma_correct(s2[0], gamma),
                              gamma_correct(s2[bpp], gamma), wx);
                   r1 = blerp(gamma_correct(s2[sw * bpp], gamma),
                              gamma_correct(s2[(sw + 1) * bpp], gamma), wx);
                   v1 = blerp(r0, r1, wy);
-                  
+
                   v = blerp(v0, v1, wz);
                   if(v < 0) v = 0;
                   if(v > 255) v = 255;
@@ -927,7 +935,7 @@ static void scale_volume_image_bilinear(unsigned char *dst, int dw, int dh, int
          }
          else
             iz = wz = 0;
-         
+
          for(y = 0; y < dh; ++y)
          {
             if(dh > 1)
@@ -939,7 +947,7 @@ static void scale_volume_image_bilinear(unsigned char *dst, int dw, int dh, int
             }
             else
                iy = wy = 0;
-            
+
             for(x = 0; x < dw; ++x)
             {
                if(dw > 1)
@@ -951,20 +959,20 @@ static void scale_volume_image_bilinear(unsigned char *dst, int dw, int dh, int
                }
                else
                   ix = wx = 0;
-               
+
                s1 = src + ((iz * (sw * sh)) + (iy * sw) + ix) * bpp;
                s2 = src + (((iz + 1) * (sw * sh)) + (iy * sw) + ix) * bpp;
-               
+
                for(n = 0; n < bpp; ++n)
                {
                   r0 = blerp(s1[0], s1[bpp], wx);
                   r1 = blerp(s1[sw * bpp], s1[(sw + 1) * bpp], wx);
                   v0 = blerp(r0, r1, wy);
-                  
+
                   r0 = blerp(s2[0], s2[bpp], wx);
                   r1 = blerp(s2[sw * bpp], s2[(sw + 1) * bpp], wx);
                   v1 = blerp(r0, r1, wy);
-                  
+
                   v = blerp(v0, v1, wz);
                   if(v < 0) v = 0;
                   if(v > 255) v = 255;
@@ -992,7 +1000,7 @@ static void scale_volume_image_cubic(unsigned char *dst, int dw, int dh, int dd,
    int dslice = dw * dh * bpp;
    unsigned char *s0, *s1, *s2, *s3;
    float invgamma;
-   
+
    if(gc)
    {
       invgamma = 1.0 / gamma;
@@ -1008,7 +1016,7 @@ static void scale_volume_image_cubic(unsigned char *dst, int dw, int dh, int dd,
          }
          else
             iz = wz = 0;
-         
+
          for(y = 0; y < dh; ++y)
          {
             if(dh > 1)
@@ -1030,14 +1038,14 @@ static void scale_volume_image_cubic(unsigned char *dst, int dw, int dh, int dd,
                   wx = ix & 0x7f;
                   ix >>= 7;
                }
-               else 
+               else
                   ix = wx = 0;
 
                s0 = src + (((iz - 1) * (sw * sh)) + ((iy - 1) * sw) + (ix - 1)) * bpp;
                s1 = s0 + sslice;
                s2 = s1 + sslice;
                s3 = s2 + sslice;
-            
+
                for(n = 0; n < bpp; ++n)
                {
                   b = icerp(gamma_correct(s1[(sw + 0) * bpp], gamma),
@@ -1053,7 +1061,7 @@ static void scale_volume_image_cubic(unsigned char *dst, int dw, int dh, int dd,
                   }
                   else
                      a = b;
-                  
+
                   c = icerp(gamma_correct(s1[(2 * sw + 0) * bpp], gamma),
                             gamma_correct(s1[(2 * sw + 1) * bpp], gamma),
                             gamma_correct(s1[(2 * sw + 2) * bpp], gamma),
@@ -1067,9 +1075,9 @@ static void scale_volume_image_cubic(unsigned char *dst, int dw, int dh, int dd,
                   }
                   else
                      d = c;
-               
+
                   v1 = icerp(a, b, c, d, wy);
-                  
+
                   if(iz > 0)
                   {
                      b = icerp(gamma_correct(s0[(sw + 0) * bpp], gamma),
@@ -1085,7 +1093,7 @@ static void scale_volume_image_cubic(unsigned char *dst, int dw, int dh, int dd,
                      }
                      else
                         a = b;
-                     
+
                      c = icerp(gamma_correct(s0[(2 * sw + 0) * bpp], gamma),
                                gamma_correct(s0[(2 * sw + 1) * bpp], gamma),
                                gamma_correct(s0[(2 * sw + 2) * bpp], gamma),
@@ -1099,7 +1107,7 @@ static void scale_volume_image_cubic(unsigned char *dst, int dw, int dh, int dd,
                      }
                      else
                         d = c;
-               
+
                      v0 = icerp(a, b, c, d, wy);
                   }
                   else
@@ -1118,7 +1126,7 @@ static void scale_volume_image_cubic(unsigned char *dst, int dw, int dh, int dd,
                   }
                   else
                      a = b;
-                  
+
                   c = icerp(gamma_correct(s2[(2 * sw + 0) * bpp], gamma),
                             gamma_correct(s2[(2 * sw + 1) * bpp], gamma),
                             gamma_correct(s2[(2 * sw + 2) * bpp], gamma),
@@ -1132,9 +1140,9 @@ static void scale_volume_image_cubic(unsigned char *dst, int dw, int dh, int dd,
                   }
                   else
                      d = c;
-                  
+
                   v2 = icerp(a, b, c, d, wy);
-                  
+
                   if(iz < dd - 1)
                   {
                      b = icerp(gamma_correct(s3[(sw + 0) * bpp], gamma),
@@ -1150,7 +1158,7 @@ static void scale_volume_image_cubic(unsigned char *dst, int dw, int dh, int dd,
                      }
                      else
                         a = b;
-                     
+
                      c = icerp(gamma_correct(s3[(2 * sw + 0) * bpp], gamma),
                                gamma_correct(s3[(2 * sw + 1) * bpp], gamma),
                                gamma_correct(s3[(2 * sw + 2) * bpp], gamma),
@@ -1164,20 +1172,20 @@ static void scale_volume_image_cubic(unsigned char *dst, int dw, int dh, int dd,
                      }
                      else
                         d = c;
-               
+
                      v3 = icerp(a, b, c, d, wy);
                   }
                   else
                      v3 = v2;
-                  
+
                   val = icerp(v0, v1, v2, v3, wz);
-                  
+
                   if(val <   0) val = 0;
                   if(val > 255) val = 255;
-                  
+
                   dst[(z * dslice) + (y * dstride) + (x * bpp) + n] =
                      gamma_correct(val, invgamma);
-                  
+
                   ++s0;
                   ++s1;
                   ++s2;
@@ -1200,7 +1208,7 @@ static void scale_volume_image_cubic(unsigned char *dst, int dw, int dh, int dd,
          }
          else
             iz = wz = 0;
-         
+
          for(y = 0; y < dh; ++y)
          {
             if(dh > 1)
@@ -1222,14 +1230,14 @@ static void scale_volume_image_cubic(unsigned char *dst, int dw, int dh, int dd,
                   wx = ix & 0x7f;
                   ix >>= 7;
                }
-               else 
+               else
                   ix = wx = 0;
 
                s0 = src + (((iz - 1) * (sw * sh)) + ((iy - 1) * sw) + (ix - 1)) * bpp;
                s1 = s0 + sslice;
                s2 = s1 + sslice;
                s3 = s2 + sslice;
-            
+
                for(n = 0; n < bpp; ++n)
                {
                   b = icerp(s1[(sw + 0) * bpp],
@@ -1245,7 +1253,7 @@ static void scale_volume_image_cubic(unsigned char *dst, int dw, int dh, int dd,
                   }
                   else
                      a = b;
-                  
+
                   c = icerp(s1[(2 * sw + 0) * bpp],
                            s1[(2 * sw + 1) * bpp],
                            s1[(2 * sw + 2) * bpp],
@@ -1259,9 +1267,9 @@ static void scale_volume_image_cubic(unsigned char *dst, int dw, int dh, int dd,
                   }
                   else
                      d = c;
-               
+
                   v1 = icerp(a, b, c, d, wy);
-                  
+
                   if(iz > 0)
                   {
                      b = icerp(s0[(sw + 0) * bpp],
@@ -1277,7 +1285,7 @@ static void scale_volume_image_cubic(unsigned char *dst, int dw, int dh, int dd,
                      }
                      else
                         a = b;
-                     
+
                      c = icerp(s0[(2 * sw + 0) * bpp],
                               s0[(2 * sw + 1) * bpp],
                               s0[(2 * sw + 2) * bpp],
@@ -1291,7 +1299,7 @@ static void scale_volume_image_cubic(unsigned char *dst, int dw, int dh, int dd,
                      }
                      else
                         d = c;
-               
+
                      v0 = icerp(a, b, c, d, wy);
                   }
                   else
@@ -1310,7 +1318,7 @@ static void scale_volume_image_cubic(unsigned char *dst, int dw, int dh, int dd,
                   }
                   else
                      a = b;
-                  
+
                   c = icerp(s2[(2 * sw + 0) * bpp],
                            s2[(2 * sw + 1) * bpp],
                            s2[(2 * sw + 2) * bpp],
@@ -1324,9 +1332,9 @@ static void scale_volume_image_cubic(unsigned char *dst, int dw, int dh, int dd,
                   }
                   else
                      d = c;
-                  
+
                   v2 = icerp(a, b, c, d, wy);
-                  
+
                   if(iz < dd - 1)
                   {
                      b = icerp(s3[(sw + 0) * bpp],
@@ -1342,7 +1350,7 @@ static void scale_volume_image_cubic(unsigned char *dst, int dw, int dh, int dd,
                      }
                      else
                         a = b;
-                     
+
                      c = icerp(s3[(2 * sw + 0) * bpp],
                               s3[(2 * sw + 1) * bpp],
                               s3[(2 * sw + 2) * bpp],
@@ -1356,19 +1364,19 @@ static void scale_volume_image_cubic(unsigned char *dst, int dw, int dh, int dd,
                      }
                      else
                         d = c;
-               
+
                      v3 = icerp(a, b, c, d, wy);
                   }
                   else
                      v3 = v2;
-                  
+
                   val = icerp(v0, v1, v2, v3, wz);
-                  
+
                   if(val <   0) val = 0;
                   if(val > 255) val = 255;
-                  
+
                   dst[(z * dslice) + (y * dstride) + (x * bpp) + n] = val;
-                  
+
                   ++s0;
                   ++s1;
                   ++s2;
@@ -1390,17 +1398,17 @@ int generate_volume_mipmaps(unsigned char *dst, unsigned char *src,
    unsigned int dw, dh, dd;
    unsigned char *s, *d;
    volmipmapfunc_t mipmap_func = NULL;
-   
+
    if(indexed)
       mipmap_func = scale_volume_image_nearest;
    else
    {
       switch(filter)
       {
-         case DDS_MIPMAP_NEAREST:  mipmap_func = scale_volume_image_nearest;  break;
-         case DDS_MIPMAP_BILINEAR: mipmap_func = scale_volume_image_bilinear; break;
-         case DDS_MIPMAP_BICUBIC:  mipmap_func = scale_volume_image_cubic;    break;
-         case DDS_MIPMAP_BOX:
+         case DDS_MIPMAP_FILTER_NEAREST:  mipmap_func = scale_volume_image_nearest;  break;
+         case DDS_MIPMAP_FILTER_BILINEAR: mipmap_func = scale_volume_image_bilinear; break;
+         case DDS_MIPMAP_FILTER_BICUBIC:  mipmap_func = scale_volume_image_cubic;    break;
+         case DDS_MIPMAP_FILTER_BOX:
          default:
                                    mipmap_func = scale_volume_image_box;      break;
       }
@@ -1410,11 +1418,11 @@ int generate_volume_mipmaps(unsigned char *dst, unsigned char *src,
 
    s = dst;
    d = dst + (width * height * depth * bpp);
-   
+
    sw = width;
    sh = height;
    sd = depth;
-   
+
    for(i = 1; i < mipmaps; ++i)
    {
       dw = MAX(1, sw >> 1);
@@ -1429,6 +1437,6 @@ int generate_volume_mipmaps(unsigned char *dst, unsigned char *src,
       sd = dd;
       d += (dw * dh * dd * bpp);
    }
-   
+
    return(1);
 }
